
##############################################################################
#
#	Pronto::Crypt::GPG -	This module provides the interfacing
#		required to get Pronto to interface to GPG.  This module
#		is defined to be as generic as possible and I encourage
#		any use for any other project to feed back any bug fixes
#		or modifications back to us so we can all benefit.
#
#	Author		-	Redvers Davies <red@madhouse.org.uk>,
#			-	Maher Awamy <muhri@muhri.net>
#	
#	Copyright 2000 Critical Integration Ltd
#	Licenced under the GNU Public Licence (GPL).
#
#	$Id: GPG.pm,v 1.6 2002/05/19 06:20:47 muhri Exp $
#
##############################################################################

package Pronto::Crypt::GPG;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
use IPC::Open3;
use MIME::Parser;

require Exporter;

#@ISA = qw(Pronto::Crypt Exporter AutoLoader);
@ISA = qw(Pronto::Crypt);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	
);
$VERSION = '0.01';

sub new {
	my $class = shift;
	my $self = {};
	my $hashref = shift;

	foreach my $key (keys %$hashref) {
		$self->{$key} = $hashref->{$key};
	}

	bless($self, $class);
	return($self);
}

## Method to see if keyid specified is in the local keyring

sub is_key_local {
	my $self = shift;
	my $pattern = shift;

	$self->wr_debug("KeyID to check is $pattern");
	$self->wr_debug("Using GPG binary at: ".$self->{gpgpath});

	$self->wr_debug("$self->{gpgpath} --fingerprint --with-colons $pattern 2> /dev/null");
	my @gpg_raw = `$self->{gpgpath} --fingerprint --with-colons $pattern 2> /dev/null`;
	chomp(@gpg_raw);

	## Return hash in following manner:
	## $rethash->{keyid}{bits,created,string,fingerprint...}

	my $counter = 0;
	my $keyid;		## Has to be sticky
	my $returnhash = {};	## Define it properly

	while ($counter < $#gpg_raw) {
		$self->wr_debug("Line: $gpg_raw[$counter]");
		if ($gpg_raw[$counter] =~ /^pub/) {
			my @public_array = split(/:/, $gpg_raw[$counter]);
			my $longid = $public_array[4];
			$self->wr_debug("Long ID: $longid");
			$longid =~ /(........)$/;
			$keyid = $1;

			$returnhash->{"0x$keyid"}{trust} = $public_array[1];
			$returnhash->{"0x$keyid"}{length} = $public_array[2];
			$returnhash->{"0x$keyid"}{algorithm} = $public_array[3];
			$returnhash->{"0x$keyid"}{fullkeyid} = $public_array[4];
			$returnhash->{"0x$keyid"}{created} = $public_array[5];
			$returnhash->{"0x$keyid"}{expires} = $public_array[6];
			$returnhash->{"0x$keyid"}{localid} = $public_array[7];
			$returnhash->{"0x$keyid"}{ownertrust} = $public_array[8];
			$returnhash->{"0x$keyid"}{string} = $public_array[9];

			$self->wr_debug("Keyid: 0x$keyid");
			$self->wr_debug("Length: $returnhash->{\"0x$keyid\"}{length}");
			$self->wr_debug("Algorithm: $returnhash->{\"0x$keyid\"}{algorithm}");
			$self->wr_debug("FullkeyID: $returnhash->{\"0x$keyid\"}{fullkeyid}");
			$self->wr_debug("Created: $returnhash->{\"0x$keyid\"}{created}");
			$self->wr_debug("Expires: $returnhash->{\"0x$keyid\"}{expires}");
			$self->wr_debug("LocalID: $returnhash->{\"0x$keyid\"}{localid}");
			$self->wr_debug("Ownertrust: $returnhash->{\"0x$keyid\"}{ownertrust}");
			$self->wr_debug("String: $returnhash->{\"0x$keyid\"}{string}");

			$counter++;
			next;
		}

		if ($gpg_raw[$counter] =~ /^fpr/) {
			my @fingerprint_array = split(/:/, $gpg_raw[$counter]);
			$self->wr_debug("0x$keyid => $fingerprint_array[9]");
			$returnhash->{"0x$keyid"}{fingerprint} = $fingerprint_array[9];
			$counter++;
			next;
		}

	$counter++;
		
	}

	return(1, $returnhash);
}

## Encryption and signing

sub encrypt {
	my $self = shift;
	my $keyarray = shift;
	my $textarray = shift;
	my $encrypt = shift;
	my $sign = shift;
	my $ownkey = shift;
	my $passphrase;
	
	if ($sign == 1) {
	
		if (defined $self->{passphrase}) {
		       $passphrase = $self->{passphrase};
	       	} else {
		       $self->obtain_passphrase();
		       $passphrase = $self->{'passphrase'};
		}	
	}
	
	my @flags;
	my $pid;

	foreach my $user (@$keyarray) {
		push(@flags, "-r \"$user\" ");
	}

	if (($encrypt) && ($sign == 1)) {
		$pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty -sea -u \"$ownkey\" @flags");
#		$self->wr_debug("$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty -sea -u $ownkey @flags");
	} elsif (($encrypt) && (!($sign == 1))) {
		$pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty -ea @flags");
#		$self->wr_debug("$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty -ea @flags");
	} elsif ((!($encrypt)) && ($sign == 1)) {
		$pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty --clearsign -u \"$ownkey\"");
#		$self->wr_debug("$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty --clearsign -u \"$ownkey\" @flags");
	} else {
		$self->wr_debug("Why were we called in the first place?");
		return 1;
	}
		
	if ($sign == 1) {
		print(WTRFH "$passphrase\n");
	}

	# Make gpg-input-handle nonblocking
	my $ioflags = fcntl(WTRFH, F_GETFL, 0)
		or die "Can't get flags for the socket: $!\n";
	fcntl(WTRFH, F_SETFL, $ioflags | O_NONBLOCK)
		or die "Can't set flags for the socket: $!\n";

	# Make gpg-output-handle nonblocking
	$ioflags = fcntl(RDRFH, F_GETFL, 0)
		or die "Can't get flags for the socket: $!\n";
	fcntl(RDRFH, F_SETFL, $ioflags | O_NONBLOCK)
		or die "Can't set flags for the socket: $!\n";

	my @stdout;
	foreach (@$textarray) {
		# There's a '\n' as the first character, this was not in the
		# original message, so let's get rid of it.
		# Addendum: I've just noticed that for some reason unknown to 
		# me it _is_ needed if the message is just encrypted?!?
		$_ =~ s/^\n// if ($sign == 1);

		# As all lines are in the first element of the given array,
		# I had to split it up somehow and couldn't find a better
		# way than this hack.
		foreach my $line_in (split(/\n/, $_, -1)) { 
			# Send line to gpg
			my $lenwritten = syswrite (WTRFH,"$line_in\n");
			# When it didn't work
			while (!$lenwritten) 
			{
				# Flush gpg-outputstream
				while (<RDRFH>) {push @stdout, "$_";}
				# Try writing again
				$lenwritten = syswrite (WTRFH,"$line_in\n");
			}
		}
	}
	# Make sure we have enough space in the outputbuffer
	# for the signature
	while (<RDRFH>) {push @stdout, "$_";}

	# Close gpg-inputhandle
	close(WTRFH);

	# Make the gpg-outputhandle blocking again.
	fcntl(RDRFH, F_SETFL, $ioflags & ~O_NONBLOCK)
		or die "Can't get flags for the socket: $!\n";

	while (<RDRFH>) {push @stdout, "$_";}
#	$self->wr_debug("STDOUT: @stdout");
	close(RDRFH);
	
	my @sterr = <ERRFH>;
#	@sterr = grep(/\[GNUPG:\]/, @sterr);
#	$self->wr_debug("STDERR: @sterr");
	close(ERRFH);

	if ($main::prefs{'forget_passphrase'}) {
	   $self->{'passphrase'} = undef;
	}   

	if (!$stdout[0] || $stdout[0] !~ /--BEGIN/) {
		return([], \@sterr);
	}

	return(\@stdout, \@sterr);
}

sub sign_detached {
	my $self = shift;
	my $ownkey = shift;
	my $textarray = shift;
	my $passphrase;
	
	if (defined $self->{passphrase}) {
	       $passphrase = $self->{passphrase};
	} else {
	       $self->obtain_passphrase();
	       $passphrase = $self->{'passphrase'};
	}	
	
	my @flags;
	my $pid;

	$pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --status-fd 2 --batch --passphrase-fd 0 --no-tty --detach-sign --armor --digest-algo SHA1 -u \"$ownkey\"");
	
	print(WTRFH "$passphrase\n");

	my $boundary="";
	while (length($boundary) < 41) {
		$boundary .= (rand > 0.5)?'-':'=';
	}

	my $stdout;
	my $oldcontenttype;
	my $oldcontenttransferenc;
	my $bodyflag = 0;
	my $contenttype = "multipart/signed;\n\tboundary=\"$boundary\";\n\tmicalg=pgp-sha1; protocol=\"application/pgp-signature\"";

	foreach (@$textarray) {
		# As all lines are in the first element of the given array,
		# I had to split it up somehow and couldn't find a better
		# way than this hack.
		foreach my $line_in (split(/\n/, $_, -1)) {

			# A dot at the beginning of a line needs to be escaped
			# as some MTAs like sendmail kill the dot otherwise
			if (($bodyflag) && ($line_in =~ /^\./)) {
				$line_in =~ s/^\./=2E/;
			}

			# parse the body through gpg and alter it accordingly
			if ($bodyflag == 1) {
				$stdout .= "$line_in\r\n";
				print(WTRFH "$line_in\r\n");
			}

			# check for a superfluous empty line and discard it
			if ($bodyflag == 2) {
				if (!($line_in =~ /^$/)) {
					$stdout .= "$line_in\r\n";
					print(WTRFH "$line_in\r\n");
				}

				$bodyflag = 1;
			}

			# change the main header
			if (not $bodyflag) {
				if ($line_in =~ /^Content-Type: /) {
					$stdout .= "Content-Type: $contenttype\n";
					$oldcontenttype = "$line_in\r\n";
				} elsif ($line_in =~ /^Content-Transfer-Encoding:/) {
					$oldcontenttransferenc = "$line_in\r\n";
				} else {
					$stdout .= "$line_in\n";
				}

				if ($line_in =~ /^$/) {
					$bodyflag = 2;
					$stdout .= "--$boundary\n";

					$stdout .= $oldcontenttype;
					$stdout .= "$oldcontenttransferenc\r\n";

					print(WTRFH "$oldcontenttype");
					print(WTRFH "$oldcontenttransferenc\r\n");
				}
			}
		}
	}

	$stdout .= "\r\n";
	$stdout .= "--$boundary\n";
	$stdout .= "Content-Type: application/pgp-signature\n\n";
	close WTRFH;
	
	my @signature = <RDRFH>;
	close RDRFH;

	foreach (@signature) {
		$stdout .= $_;
	}
	$stdout .= "\n--$boundary--\n";

	# Convert \r\n back to \n as some MUAs don't seem to like them. For
	# verification they are required to add them again if needed, anyway.
	$stdout =~ s/\x0D\x0A/\x0A/g;
	
	my @sterr = <ERRFH>;
	close ERRFH;
#	$self->wr_debug("sign_detached: $stdout");

	return ($contenttype, $stdout, \@sterr);
}

## Decryption

sub decrypt {
	my $self = shift;
	my $messageref = shift;
	
	# create a new MIME entity
	my $parser = new MIME::Parser;
	$parser->output_to_core(1);
	$parser->ignore_errors(1);
	my $entity = eval { $parser->parse_data($messageref) };

	# I really should do something with errors...
	my $error = ($@ || $parser->last_error);

	my $stdout;
	my $sterr;

	if (($entity->head->mime_type eq "multipart/encrypted") &&
	    ($entity->parts(0)->bodyhandle->as_string =~ /Version: 1/i) &&
    	    ($entity->parts(1)->head->mime_type eq "application/octet-stream")) {
		# This is a RFC 2015 message, special treatment is required.
		# (I know, the above test may be a bit paranoidac.)

		my $encrypteddata = $entity->parts(1)->bodyhandle->as_string;
		(my $decrypteddata, $sterr) = &decryptdata($self, $encrypteddata);

		if (@$sterr[1] eq "BAD_PASSPHRASE") {
			return ([], [@$sterr[0]]);
		}

		# Set up a new entity for the decoded message
		my $entity2 = eval { $parser->parse_data(@$decrypteddata) };

		# And create a new message out of the original and the decoded data
		if ($entity2->head->get('Mime-Version')) {
			$entity->head->replace("Mime-Version", $entity2->head->get('Mime-Version'));
		}
		if ($entity2->head->get('Content-Type')) {
			$entity->head->replace("Content-Type", $entity2->head->get('Content-Type'));
		}
		if ($entity2->head->get('Content-Transfer-Encoding')) {
			$entity->head->replace("Content-Transfer-Encoding", $entity2->head->get('Content-Transfer-Encoding'));
		}
		@$stdout[0] = $entity->head->as_string;
		
		# I cannot use the MIME::Tools further, as the data may be encrypted
		# and the MIME:Tools sometimes change the content of a message. :-(
		my $bodyflag = 0;
		foreach my $line (split(/\n/, @$decrypteddata[0], -1)) { 
			if ($line =~ /^$/) { $bodyflag = 1; }
			if ($bodyflag) {
				@$stdout[0] .= "$line\n";
			}
		}

		# It is possible, that the message contains, now that it is
		# decrypted, a detached signature. Due to Pronto's message
		# handling and the restrictions with the MIME::Tools and
		# signatures I have to check for such a case here and do the
		# verification right now.
		
		if ($entity->head->mime_attr("content-type.protocol")) {
			if ($entity->head->mime_attr("content-type.protocol") =~ /application\/pgp-signature/i) {
				my ($stdout2, $sterr2) = &verify_detached($self, @$stdout[0]);
				push @$sterr, _("\nThis message contained a detached signature, below is the result of its verification:\n\n");
				push @$sterr, @$stdout2;
				push @$sterr, @$sterr2;
			}
		}
	}
	elsif ($messageref =~ /-----BEGIN PGP MESSAGE-----/) {
		# There seems to be a "normally" encrypted part somewhere

		# Find all parts with data to be decrypted
		my $encrypteddata = &decrypt_recurse_helper($self, $entity);

		# decrypt and replace it
		my $message = $entity->stringify;
		my $decrypted_something = 0;
		foreach my $datapart (@$encrypteddata) {
			my ($plain, $err) = &decryptdata($self, $datapart);
			if (@$err[1] eq "BAD_PASSPHRASE") {
				return ([], [@$err[0]]);
			}

			if ((@$plain) && (@$plain[0] ne "")) {
				$message =~ s/\Q$datapart\E/@$plain[0]/;
				$decrypted_something = 1;
			}
			push @$sterr, @$err;
		}
		@$stdout[0] = $message;
	
		@$sterr = _("Error: No encrypted data found.") unless ($decrypted_something);
	}
	else {
		# I don't think that this is an encrypted message

		@$sterr = _("Error: No encrypted data found.");
	}

	return(\@$stdout, \@$sterr);
}

sub decrypt_recurse_helper {
	my $self   = shift;
	my $entity = shift;
	my $encrypteddata = shift;

	if ($entity->parts == 0) {
		if (($entity->bodyhandle->as_string =~ /-----BEGIN PGP MESSAGE-----/) &&
		   !($entity->head->recommended_filename)) {
			push @$encrypteddata, $entity->bodyhandle->as_string;
		}
	} else {
		foreach my $part ($entity->parts) {
			$encrypteddata = &decrypt_recurse_helper($self, $part, $encrypteddata);
		}
	}

	return(\@$encrypteddata);
}

sub decryptdata {
	my $self = shift;
	my $encrypteddata = shift;
	my $passphrase = $self->{'passphrase'};

	if (!$passphrase) {
		$self->obtain_passphrase();
		$passphrase = $self->{'passphrase'};
	}	

	my $pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --decrypt --batch --passphrase-fd 0 --status-fd 2 --no-tty");

	print(WTRFH "$passphrase\n");
	print WTRFH $encrypteddata;
	
	close WTRFH;
	
	#my @stdout = <RDRFH>; # This version doesn't work, I get funny spaces in front of each line.
	my @stdout;
	foreach my $line (<RDRFH>) {
		$stdout[0] .= $line;
	}
	close RDRFH;
	
	my @sterr = <ERRFH>;
	close ERRFH;

	if (grep(/BAD_PASSPHRASE/, @sterr)) {
		$self->{'passphrase'} = undef;
		@stdout = undef;
		my @userid_hint = grep(/\[GNUPG:\]\ USERID_HINT/, @sterr);
		(my $ownkey = $userid_hint[0]) =~ s/.*USERID_HINT [A-Z0-9]* (.*?)\n/$1/s;
		@sterr  = undef;
		$sterr[0] = _("You entered a bad passphrase for your key \"$ownkey\".\n");
		$sterr[1] = "BAD_PASSPHRASE";
	}
	@sterr = grep(!/\[GNUPG:\]/, @sterr);

	if ($main::prefs{'forget_passphrase'}) {
	   $self->{'passphrase'} = undef;
	}   

	if (!$stdout[0] || $stdout[0] eq " ") {
		return ([], \@sterr);
	}	

       
	return(\@stdout, \@sterr);
}

## Verification of signatures

sub verify
{
	if ($_[1] =~ /application\/x-pkcs7-signature/i) {
		if (-f $main::prefs{'opensslpath'}) {
			return(&verify_pkcs7(@_));
		}
		else {
			return([_("Location of OpenSSL was not specified.")],[_("Please enter the path in the options dialog.")]);
		}
	}
	if ($_[1] =~ /\nContent-Type: application\/pgp-signature/i) {
		return(&verify_detached(@_));
	} elsif ($_[1] =~ /-----BEGIN PGP SIGNED MESSAGE-----/) {
		return(&verify_single(@_));
	} else {
		return ([""], [_("Error: No signed data found.")]);
	}
}

sub verify_single
{
	my $self = shift;
	my $messageref = shift;
	
	# Remove quoted-printable encoding if necessary
	if ($messageref =~ /=2D----BEGIN PGP SIGNED MESSAGE-----/i ) {
		$messageref =~ s/=([a-fA-F0-9]{2})/chr hex $1/ge;
		$messageref =~ s/=\n//g;
	}

	my $pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --verify --status-fd 2 --batch --no-tty");

	print WTRFH $messageref;
	
	close WTRFH;
	
	my @stdout = <RDRFH>;
	close RDRFH;
	
	my @sterr = <ERRFH>;
	my @nokey = grep(/\[GNUPG:\]\ NO_PUBKEY/, @sterr);
	@sterr = grep(!/\[GNUPG:\]/, @sterr);
	close ERRFH;
	
	if ($nokey[0]) {
		(my $requiredkey = $nokey[0]) =~ s/.*PUBKEY\ ([A-Z0-9]*).*/$1/s;
		if (&receive_key($self, $requiredkey)) {
			return(&verify_single($self, $messageref));
		}
	}

	if (!$stdout[0] || $stdout[0] eq " ") {
		return([], \@sterr);
	}	
	
	return(\@stdout, \@sterr);
}

sub verify_detached
{
	my $self = shift;
	my $messageref = shift;

	# Original code taken from pgp-mime-handler.pl by 
	# David Gmbel <david.guembel@gmx.de>

	# create a new MIME entity
	my $parser = new MIME::Parser;
	$parser->output_to_core(1);
	$parser->ignore_errors(1);
	my $entity = eval { $parser->parse_data($messageref) };
	# I really should do something with errors...
	my $error = ($@ || $parser->last_error);

	# according to RFC 2015, multipart/signed messages MUST consist of
	# exactly two parts:
	my $part_1 = $entity->parts(0);
	my $part_2 = $entity->parts(1);

	# It's a shame that I cannot use $part_1, but the MIME object tends to
	# change line breaks which would invalidate a signature.
	my $boundary = $entity->head->multipart_boundary;
	$messageref =~ s/\x0D\x0A/\x0A/g;
	(my $toverify = $messageref) =~ s/.*?\n\-\-\Q$boundary\E\n(.+?)\n\-\-\Q$boundary\E.*/$1/s;
	$toverify =~ s/\x0A/\x0D\x0A/g;

	open SIG, ">".$main::prefs{'MailDir'}."/tmp/signature";
	print SIG $part_2->as_string;
	close SIG;

	my $pid = open3(\*WTRFH, \*RDRFH, \*ERRFH, "$self->{gpgpath} --charset iso-8859-1 --verify --status-fd 2 --batch --no-tty ".$main::prefs{'MailDir'}."/tmp/signature -");

	print WTRFH $toverify;
	close WTRFH;

	my @stdout = <RDRFH>;
	close RDRFH;
	
	my @sterr = <ERRFH>;
	my @nokey = grep(/\[GNUPG:\]\ NO_PUBKEY/, @sterr);
	@sterr = grep(!/\[GNUPG:\]/, @sterr);
	close ERRFH;
	
	unlink $main::prefs{'MailDir'}."/tmp/signature";

	if ($nokey[0]) {
		(my $requiredkey = $nokey[0]) =~ s/.*PUBKEY\ ([A-Z0-9]*).*/$1/s;
		if (&receive_key($self, $requiredkey)) {
			return(&verify_detached($self, $messageref));
		}
	}

	if (!$stdout[0] || $stdout[0] eq " ") {
		return([], \@sterr);
	}	
	
	return(\@stdout, \@sterr);
}

sub verify_pkcs7 {
	my $self = shift;
	my $messageref = shift;

	my @stdout = `$main::prefs{'opensslpath'} smime -verify -in $main::prefs{'MailDir'}/tmp/dump -out /dev/null 2>&1`;

	return([], \@stdout);
}

## Several helper functions

sub receive_key {
	my $self = shift;
	my $keyid = shift;

	if (&Pronto::Crypt::GUI::ask_for_search()) {
		my @getresult = `$self->{gpgpath} --status-fd 2 --recv-keys $keyid 2>&1`;

		my $nodata = 0;
		if (grep(/\[GNUPG:\]\ NODATA/, @getresult)) {
			$nodata = 1;
		}
		@getresult = grep(!/\[GNUPG:\]/, @getresult);
	
		if ($nodata) {
			&main::err_dialog(_("The required key wasn't found."),"y",_("Receive key failure"));
			return 0;
		}
		else {
			&main::err_dialog("@getresult","y",_("Receive key successful"));
			return 1;
		}
	}
	return 0;
}

sub search_server {
	my $self = shift;
	my $server = shift;
	my $string = shift;

	$self->wr_debug("search_server\($server, $string\)");
	my $teststring = "http://wwwkeys.pgp.net:11371/pks/lookup?op=vindex&search=redvers&fingerprint=on";
	
	my $serverobj = new IO::Socket::INET(PeerAddr => "$server", PeerPort => "11371", Proto => "tcp");
	print($serverobj "GET http://wwwkeys.pgp.net:11371/pks/lookup?op=vindex&search=redvers&fingerprint=on");
	print($serverobj "\n\n");

	my @raw_server = <$serverobj>;

	close($serverobj);

	$self->wr_debug("@raw_server");
}

sub set_path {
	my $self = shift;
	my $path = shift;

	$self->{gpgpath} = $path;

	return 1;
}


















#objectref->import_key_from_server
#objectref->send_key_to_server
#objectref->localsign
#objectref->realsign

# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Pronto::Crypt::GPG - Perl extension for blah blah blah

=head1 SYNOPSIS

  use Pronto::Crypt::GPG;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Pronto::Crypt::GPG was created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head1 AUTHOR

A. U. Thor, a.u.thor@a.galaxy.far.far.away

=head1 SEE ALSO

perl(1).

=cut


## Method new: Used to initialise the GPG support.

